@@ -445,6 +445,12 @@ chkspd(x) int x; {
 	case 2400:
 	case 4800:
 	case 9600:
+#ifdef MINIX
+        case 19200:
+	case 38400:
+	case 57600:
+	case 115200:
+#else
 #ifdef apollo
         case 19200:
 #else
@@ -464,6 +470,7 @@ chkspd(x) int x; {
 #endif
 #endif
 #endif
+#endif
 
 #ifdef datageneral
 	case 50:
