@@ -42,7 +42,7 @@ PUBLIC _PROTOTYPE (int (*call_vector[NCALLS]), (void) ) = {
 	do_set,		/* 23 = setuid	*/
 	no_sys,		/* 24 = getuid	*/
 	do_stime,	/* 25 = stime	*/
-	no_sys,		/* 26 = (ptrace)*/
+	no_sys,		/* 26 = ptrace	*/
 	no_sys,		/* 27 = alarm	*/
 	do_fstat,	/* 28 = fstat	*/
 	no_sys,		/* 29 = pause	*/
@@ -64,7 +64,7 @@ PUBLIC _PROTOTYPE (int (*call_vector[NCALLS]), (void) ) = {
 	no_sys,		/* 45 = unused	*/
 	do_set,		/* 46 = setgid	*/
 	no_sys,		/* 47 = getgid	*/
-	no_sys,		/* 48 = sig	*/
+	no_sys,		/* 48 = (signal)*/
 	no_sys,		/* 49 = unused	*/
 	no_sys,		/* 50 = unused	*/
 	no_sys,		/* 51 = (acct)	*/
