@@ -1,8 +1,9 @@
 /* This file contains a collection of miscellaneous procedures:
  *	mem_init:	initialize memory tables.  Some memory is reported
  *			by the BIOS, some is guesstimated and checked later
- *	do_vrdwt:	unpack an i/o vector for those block device drivers
- *			which do not do it for themself
+ *	env_parse	parse environment variable.
+ *	bad_assertion	for debugging
+ *	bad_compare	for debugging
  */
 
 #include "kernel.h"
@@ -62,7 +63,6 @@ PUBLIC void mem_init()
 }
 #endif /* (CHIP == INTEL) */
 
-
 /*=========================================================================*
  *				env_parse				   *
  *=========================================================================*/
@@ -145,7 +145,6 @@ char *what;
   panic(NULL, NO_NUM);
 }
 
-
 /*=========================================================================*
  *				bad_compare				   *
  *=========================================================================*/
