@@ -3,7 +3,7 @@
   See the copyright notice in the ACK home directory, in the file "Copyright".
 */
 
-/* $Header: ngf4.c,v 1.6 89/10/25 17:15:11 ceriel Exp $ */
+/* $Header: ngf4.c,v 1.7 93/01/05 12:05:57 ceriel Exp $ */
 
 /*
 		NEGATE A FLOATING POINT (NGF 4)
@@ -14,7 +14,7 @@
 #include "get_put.h"
 
 #define OFF ((FL_MSW_AT_LOW_ADDRESS ? 0 : 2) + (FL_MSB_AT_LOW_ADDRESS ? 0 : 1))
-SINGLE
+void
 ngf4(f)
 SINGLE	f;
 {
@@ -24,5 +24,4 @@ SINGLE	f;
 		p = (unsigned char *) &f + OFF;
 		*p ^= 0x80;
 	}
-	return f;
 }
