@@ -4,8 +4,8 @@
 #include <string.h>
 
 void bcopy(src, dst, length)
-_CONST char *src;
-char *dst;
+_CONST void *src;
+void *dst;
 int length;
 {
   (void) memcpy((_VOIDSTAR) dst, (_CONST _VOIDSTAR) src, (_SIZET) length);
